package com.nisum.webfluxmongodb.controller;


import com.nisum.webfluxmongodb.dto.ProductDto;
import com.nisum.webfluxmongodb.service.ProductService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import static org.apache.commons.lang3.StringUtils.isBlank;

@RestController
@RequestMapping("/products")
public class ProductController
{

    @Autowired
    private ProductService service;

    @GetMapping
    public Flux<ProductDto> getProducts(){
        return service.getProducts();
    }

    @GetMapping("/{id}")
    public Mono<ProductDto> getProduct(@PathVariable String id){
        return service.getProduct(id);
    }

    @GetMapping("/product-range")
    public Flux<ProductDto> getProductBetweenRange(@RequestParam("min") double min, @RequestParam("max")double max){
        return service.getProductInRange(min,max);
    }

    @PostMapping
    public Mono<ProductDto> saveProduct(@RequestBody Mono<ProductDto> productDtoMono){
        System.out.println("controller method called ...");
        return service.saveProduct(productDtoMono);
    }

    @PutMapping("/update/{id}")
    public Mono<ProductDto> updateProduct(@RequestBody Mono<ProductDto> productDtoMono,@PathVariable String id){
        return service.updateProduct(productDtoMono,id);
    }

    @DeleteMapping("/delete/{id}")
    public Mono<Void> deleteProduct(@PathVariable String id){
        return service.deleteProduct(id);
    }


    @GetMapping("/search")
    public Mono<ProductDto> getSearchAllProducts(@PathVariable(name="quantity") int qty, @PathVariable(name="name") String name, @PathVariable(name="price") Double price) {
        if(null != name || !isBlank(name)){
            return service.getProductsByName(name);
        }else if(qty>0){
            return service.getProductByQuantity(qty);
        }
        return service.getProductByPrice(price);
    }




}