package com.nisum.webfluxmongodb.service;

import com.nisum.webfluxmongodb.dto.ProductDto;
import com.nisum.webfluxmongodb.repository.ProductRepository;
import com.nisum.webfluxmongodb.utils.AppUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Range;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.RequestParam;
import reactor.core.CoreSubscriber;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.util.List;

@Service
public class ProductService
{

    @Autowired
    private ProductRepository repository;


    public Flux<ProductDto> getProducts()
    {

        return repository.findAll().map(AppUtils::entityToDto);
    }

    public Mono<ProductDto> getProduct(String id)
    {

        return repository.findById(id).map(AppUtils::entityToDto);
    }

    public Flux<ProductDto> getProductInRange(double min,double max)
    {
        return repository.findByPriceBetween(Range.closed(min,max));
    }

    public Mono<ProductDto> saveProduct(Mono<ProductDto> productDtoMono)
    {
        System.out.println("service method called ...");
        return  productDtoMono.map(AppUtils::dtoToEntity)
                .flatMap(repository::insert)
                .map(AppUtils::entityToDto);
    }

    public Mono<ProductDto> updateProduct(Mono<ProductDto> productDtoMono,String id)
    {
        return repository.findById(id)
                .flatMap(p->productDtoMono.map(AppUtils::dtoToEntity)
                        .doOnNext(e->e.setId(id)))
                .flatMap(repository::save)
                .map(AppUtils::entityToDto);
    }

    public Mono<Void> deleteProduct(String id)
    {
        return repository.deleteById(id);
    }

   public Mono<ProductDto> getProductsByName(@RequestParam String name){
       return (Mono<ProductDto>) repository.findByName(name);

    }
    public Mono<ProductDto> getProductByPrice(@RequestParam double price){
        return (repository.findByPrice(price));
    }
    public Mono<ProductDto>getProductByQuantity(@RequestParam int qty){
        return repository.findByQuantity(qty) ;
//      Mono<ProductDto> productDtoMono = repository.findByQuantity(qty);
//       Mono<ProductDto> productDtoMono1=productDtoMono.map(productDto -> productDto.getQty());
//       return productDtoMono1;
    }
}